@@ -74,9 +74,6 @@ static int read_error_count;
 /* Have we hit EOF yet?  */
 static bool hit_eof;
 
-/* Checkpointing counter */
-static unsigned checkpoint;
-
 static bool read_full_records = false;
 
 /* We're reading, but we just read the last block and it's time to update.
@@ -204,7 +201,8 @@ enum compress_type {
   ct_none,
   ct_compress,
   ct_gzip,
-  ct_bzip2
+  ct_bzip2,
+  ct_lzma
 };
 
 struct zip_magic
@@ -221,6 +219,7 @@ static struct zip_magic const magic[] = {
   { ct_compress, 2, "\037\235", "compress", "-Z" },
   { ct_gzip,     2, "\037\213", "gzip", "-z"  },
   { ct_bzip2,    3, "BZh",      "bzip2", "-j" },
+  { ct_lzma,     6, "\xFFLZMA", "lzma", "--lzma" }, /* FIXME: ???? */
 };
 
 #define NMAGIC (sizeof(magic)/sizeof(magic[0]))
@@ -230,19 +229,21 @@ static struct zip_magic const magic[] = {
 
 /* Check if the file ARCHIVE is a compressed archive. */
 enum compress_type
-check_compressed_archive ()
+check_compressed_archive (bool *pshort)
 {
   struct zip_magic const *p;
   bool sfr;
-  bool short_file = false;
+  bool temp;
+
+  if (!pshort)
+    pshort = &temp;
   
   /* Prepare global data needed for find_next_block: */
   record_end = record_start; /* set up for 1st record = # 0 */
   sfr = read_full_records;
   read_full_records = true; /* Suppress fatal error on reading a partial
 			       record */
-  if (find_next_block () == 0)
-    short_file = true;
+  *pshort = find_next_block () == 0;
   
   /* Restore global values */
   read_full_records = sfr;
@@ -255,9 +256,6 @@ check_compressed_archive ()
     if (memcmp (record_start->buffer, p->magic, p->length) == 0)
       return p->type;
 
-  if (short_file)
-    ERROR ((0, 0, _("This does not look like a tar archive")));
-
   return ct_none;
 }
 
@@ -274,11 +272,16 @@ open_compressed_archive ()
 
   if (!multi_volume_option)
     {
-      enum compress_type type = check_compressed_archive ();
+      bool shortfile;
+      enum compress_type type = check_compressed_archive (&shortfile);
 
       if (type == ct_none)
-	return archive;
-
+	{
+	  if (shortfile)
+	    ERROR ((0, 0, _("This does not look like a tar archive")));
+	  return archive;
+	}
+      
       /* FD is not needed any more */
       rmtclose (archive);
 
@@ -503,15 +506,18 @@ _open_archive (enum access_mode wanted_access)
 	{
 	case ACCESS_READ:
 	  {
+	    bool shortfile;
 	    enum compress_type type;
 
 	    archive = STDIN_FILENO;
 
-	    type = check_compressed_archive ();
+	    type = check_compressed_archive (&shortfile);
 	    if (type != ct_none)
 	      FATAL_ERROR ((0, 0,
 			    _("Archive is compressed. Use %s option"),
 			    compress_option (type)));
+	    if (shortfile)
+	      ERROR ((0, 0, _("This does not look like a tar archive")));
 	  }
 	  break;
 
@@ -555,7 +561,7 @@ _open_archive (enum access_mode wanted_access)
 			   O_RDWR | O_CREAT | O_BINARY,
 			   MODE_RW, rsh_command_option);
 
-	if (check_compressed_archive () != ct_none)
+	if (check_compressed_archive (NULL) != ct_none)
 	  FATAL_ERROR ((0, 0,
 			_("Cannot update compressed archives")));
 	break;
@@ -589,43 +595,13 @@ _open_archive (enum access_mode wanted_access)
     }
 }
 
-static void
-do_checkpoint (bool do_write)
-{
-  if (checkpoint_option && !(++checkpoint % checkpoint_option))
-    {
-      switch (checkpoint_style)
-	{
-	case checkpoint_dot:
-	  fputc ('.', stdlis);
-	  fflush (stdlis);
-	  break;
-
-	case checkpoint_text:
-	  if (do_write)
-	    /* TRANSLATORS: This is a ``checkpoint of write operation'',
-	     *not* ``Writing a checkpoint''.
-	     E.g. in Spanish ``Punto de comprobaci@'on de escritura'',
-	     *not* ``Escribiendo un punto de comprobaci@'on'' */
-	    WARN ((0, 0, _("Write checkpoint %u"), checkpoint));
-	  else
-	    /* TRANSLATORS: This is a ``checkpoint of read operation'',
-	       *not* ``Reading a checkpoint''.
-	       E.g. in Spanish ``Punto de comprobaci@'on de lectura'',
-	       *not* ``Leyendo un punto de comprobaci@'on'' */
-	    WARN ((0, 0, _("Read checkpoint %u"), checkpoint));
-	  break;
-	}
-    }
-}  
-
 /* Perform a write to flush the buffer.  */
 ssize_t
 _flush_write (void)
 {
   ssize_t status;
 
-  do_checkpoint (true);
+  checkpoint_run (true);
   if (tape_length_option && tape_length_option <= bytes_written)
     {
       errno = ENOSPC;
@@ -635,7 +611,7 @@ _flush_write (void)
     status = record_size;
   else
     status = sys_write_archive_buffer ();
-
+  
   return status;
 }
 
@@ -857,7 +833,7 @@ close_archive (void)
     verify_volume ();
 
   if (rmtclose (archive) != 0)
-    close_warn (*archive_name_cursor);
+    close_error (*archive_name_cursor);
 
   sys_wait_for_child (child_pid);
 
@@ -1045,7 +1021,7 @@ new_volume (enum access_mode mode)
   current_block = record_start;
   
   if (rmtclose (archive) != 0)
-    close_warn (*archive_name_cursor);
+    close_error (*archive_name_cursor);
 
   archive_name_cursor++;
   if (archive_name_cursor == archive_name_array + archive_names)
@@ -1464,7 +1440,7 @@ simple_flush_read (void)
 {
   size_t status;		/* result from system call */
 
-  do_checkpoint (false);
+  checkpoint_run (false);
   
   /* Clear the count of errors.  This only applies to a single call to
      flush_read.  */
@@ -1523,7 +1499,7 @@ _gnu_flush_read (void)
 {
   size_t status;		/* result from system call */
 
-  do_checkpoint (false);
+  checkpoint_run (false);
   
   /* Clear the count of errors.  This only applies to a single call to
      flush_read.  */
